# Copyright (C) 2005, Timothy R. Morgan
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice and this permission
# notice appear in all copies, modified copies and in
# supporting documentation.
"React Application Framework"

from OFS.Folder import Folder, manage_addFolder
from Globals import InitializeClass, DTMLFile, HTMLFile
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from OFS.DTMLDocument import DTMLDocument
from new import function
from AccessControl import ClassSecurityInfo
from Dispatcher import Dispatcher
import os

# we'll need this later to grab some page templates and such
this_dir = os.path.split(__file__)[0]
www_dir = os.path.join(this_dir, 'www')

# get the version number from the version.txt file
__version__ = file(os.path.join(this_dir, 'version.txt'), 'r').read().strip()

class ReactApplicationFolder(Folder):
  "React Application Folder"

  meta_type = 'React Application Folder'
  
  security = ClassSecurityInfo()

  _properties = Folder._properties + (
    {'id': 'default_controller', 'type': 'string', 'mode': 'w'},
    {'id': 'default_action', 'type': 'string', 'mode': 'w'},
  )
  
  manage_options = Folder.manage_options + (
    {'label':'Upgrade', 'action':'manage_upgrade_form'},
    {'label':'Getting Started', 'action':'manage_getting_started'},
  )
  
  manage_getting_started = DTMLFile('help/getting_started', globals())
  
  manage_upgrade_form = DTMLFile('www/ReactUpgrade', globals())
  
  def manage_upgrade(self, overwrite_layouts=0, REQUEST=None):
    """
    Upgrade the layouts and Ajax JavaScript library.
    """
    layouts = ['application', 'mail', 'plone']
    # delete current layouts if specified
    for layout in layouts:
      if hasattr(self.views.layouts, layout) and overwrite_layouts:
        self.views.layouts._delObject(layout)
    # replace missing layouts
    for layout in layouts:
      if not hasattr(self.views.layouts, layout):
        self.views.layouts._setObject(layout, zpt_from_file(layout))
    # delete and re-add the Ajax library
    self.public.javascripts._delObject('prototype.js')
    self.public.javascripts._setObject('prototype.js', dtml_from_file('prototype.js'))
    # return to upgrade form
    if REQUEST:
      return REQUEST.RESPONSE.redirect('manage_upgrade_form?done=1')
  
  def __init__(self, id=None):
    Folder.__init__(self, id)
    # set up the initial property values
    self.default_controller = 'home'
    self.default_action = 'index'
    # set up the folder structure
    self._setObject('controllers', Folder('controllers'))
    self._setObject('helpers', Folder('helpers'))
    self._setObject('models', Folder('models'))
    self._setObject('public', Folder('public'))
    self.public._setObject('images', Folder('images'))
    self.public._setObject('javascripts', Folder('javascripts'))
    self.public._setObject('stylesheets', Folder('stylesheets'))
    self._setObject('views', Folder('views'))
    self.views._setObject('layouts', Folder('layouts'))
    # create some blank public files and get ready to do some Ajax using Prototype
    self.public.javascripts._setObject('global.js', dtml_from_file('global.js'))
    self.public.javascripts._setObject('prototype.js', dtml_from_file('prototype.js'))
    self.public.stylesheets._setObject('global.css', dtml_from_file('global.css'))
    # create a few initial layouts
    self.views.layouts._setObject('application', zpt_from_file('application'))
    self.views.layouts._setObject('mail', zpt_from_file('mail'))
    self.views.layouts._setObject('plone', zpt_from_file('plone'))
  
  def __bobo_traverse__(self, request, entry_name):
    """Intercept normal traversing and let the Dispatcher do some magic."""
    if hasattr(self.aq_explicit, entry_name):
      # normal traversal because the item really exists
      # this allows the management pages to work
      return getattr(self, entry_name)
    else:
      # get the path being requested
      path = request['TraversalRequestNameStack'][:]
      path.append(entry_name) # include the first part
      path.reverse() # TraversalRequestNameStack is backwards
      if path[0] == 'index_html': # we'll let Dispatcher use the default controller+view
        path = []
      if not path or hasattr(self.controllers.aq_explicit, path[0]) or hasattr(self.views.aq_explicit, path[0]) or hasattr(self.public.aq_explicit, path[0]):
        # create a Dispatcher to handle special requests
        request['TraversalRequestNameStack'] = [] # clear the stack so Zope won't keep going
        return Dispatcher(self, request, path) # let Dispatcher do its magic
      else:
        # do some acquisition (ask the parent for the item)
        return getattr(self.aq_parent, entry_name)
  
  _mailer = DTMLFile('www/mailer', globals())

InitializeClass(ReactApplicationFolder)

_code = manage_addFolder.func_code
_globals = manage_addFolder.func_globals.copy()
_globals['Folder'] = ReactApplicationFolder
manage_addReactApplicationFolder = function(
  _code, _globals, 'manage_addReactApplicationFolder',
  manage_addFolder.func_defaults
)

manage_addReactApplicationFolderForm = DTMLFile('www/ReactFolderAdd', globals())


def zpt_from_file(name):
  """Creates a ZODB Page Template from a file in the 'www' directory."""
  name += '.zpt'
  content = file(os.path.join(www_dir, name), 'r').read()
  return ZopePageTemplate(name, content, 'text/html')

def dtml_from_file(name):
  """Creates a ZODB DTML Document from a file in the 'www' directory."""
  name += '.dtml'
  content = file(os.path.join(www_dir, name), 'r').read()
  dtml = DTMLDocument()
  dtml.manage_edit(content, title='')
  return dtml
